import struct
import socket

class RequestResponse:
	s = None

	def enum(**enums):
		return type('Enum', (), enums)

	MessageType = enum(	GOTO_STATE                 = 0x01,
						CONFIG_CAMERA              = 0x02,
						CONFIG_SCAN		           = 0x03,
						GET_IMAGE                  = 0x04,
						SCAN_DATA                  = 0x05,
						SCAN_ERROR				   = 0x06,

						RESPONSE                   = 0x80)

	def __init__(self, socket):
		self.s = socket

	def sendMessage(self, messageType, data):
		msg = bytearray()
		msg.append(messageType)

		length = 0

		if data != None:
			if (type(data) == int):
				length = 4
				data = struct.pack("I", data)
			elif (type(data) == str):
				length = len(data)
			else:
				raise("Unknown data type")
	
		encoded_length = struct.pack("I", length)
		msg += encoded_length
	
		if data != None:
			msg += data

		self.s.send(msg)

	def sendResponse(self, retCode, messageType, data):
		msg = bytearray()
		msg.append(LineScanner.MessageType.RESPONSE)

		length = 2

		if data != None:
			if (type(data) == int):
				length += 4
				data = struct.pack("I", data)
			elif (type(data) == str):
				length += len(data)
			else:
				raise ValueError("Unknown data type", type(data))

		encoded_length = struct.pack("I", length)
		msg += encoded_length

		msg.append(messageType)
		msg.append(retCode)
	
		if data != None:
			msg += data

		self.s.send(msg)

	def getMessage(self):
		header = self.s.recv(5)
		header = struct.unpack('=BI', header)

		recv_type     = int(header[0])
		recv_length   = int(header[1])

		data = ''

		if recv_length > 0:
			while len(data) < recv_length:
				data += self.s.recv( min(1024, recv_length - len(data)) )		

		return (recv_type, data)

	def getScanData(self):
		(recv_type, data) = self.getMessage()
		if recv_type == self.MessageType.SCAN_ERROR:
			raise Exception("Scan error received")

		elif recv_type == self.MessageType.SCAN_DATA:
			config 	       = data[0:48]
			scan_mode      = struct.unpack('=I', config[0:4])[0]

			max_height           = struct.unpack('=I', config[4:8])[0]
			scan_lines_per_image = struct.unpack('=I', config[8:12])[0]
			scanner_ready_pin    = struct.unpack('=I', config[12:16])[0]
			fps				     = struct.unpack('=f', config[16:20])[0]
			cycle_time_diff_threshold_ms = struct.unpack('=f', config[20:24])[0]
			implementation = config[24:]
			
			scatter_distance  = struct.unpack('=I', implementation[0:4])[0]
			threshold   	  = struct.unpack('=I', implementation[4:8])[0]
			pixel_power       = struct.unpack('=f', implementation[8:12])[0]
			num_streams    	  = struct.unpack('=I', implementation[12:16])[0]
			threads_per_block = struct.unpack('=I', implementation[16:20])[0]
			num_frames         = struct.unpack('=I', implementation[20:24])[0]

			currentFieldIndex = len(config)

			height = []
			intensity = []
			scatter = []
			reflectance = []

			height = data[currentFieldIndex:currentFieldIndex+2*scan_lines_per_image*num_frames]
			height = struct.unpack('%sH' % scan_lines_per_image*num_frames, height)
			currentFieldIndex += 2*scan_lines_per_image*num_frames

			intensity = data[currentFieldIndex:currentFieldIndex+2*scan_lines_per_image*num_frames]
			intensity = struct.unpack('%sH' % scan_lines_per_image*num_frames, intensity)
			currentFieldIndex += 2*scan_lines_per_image*num_frames

			scatter = data[currentFieldIndex:currentFieldIndex+2*scan_lines_per_image*num_frames]
			scatter = struct.unpack('%sH' % scan_lines_per_image*num_frames, scatter)
			currentFieldIndex += 2*scan_lines_per_image*num_frames

			reflectance = data[currentFieldIndex:currentFieldIndex+2*scan_lines_per_image*num_frames]
			reflectance = struct.unpack('%sH' % scan_lines_per_image*num_frames, reflectance)
			currentFieldIndex += 2*scan_lines_per_image*num_frames

			color = data[currentFieldIndex:currentFieldIndex+6*scan_lines_per_image*num_frames]
			color = struct.unpack('%sH' % 3*scan_lines_per_image*num_frames, color)
			currentFieldIndex += 6*scan_lines_per_image*num_frames

			kernel_exec_time = data[currentFieldIndex:currentFieldIndex+4]
			kernel_exec_time = struct.unpack('f', kernel_exec_time)[0]
			currentFieldIndex += 4

			total_exec_time = data[currentFieldIndex:currentFieldIndex+4]
			total_exec_time = struct.unpack('f', total_exec_time)[0]
			currentFieldIndex += 4

			for i in range(num_frames):
				if (i == 0):
					yield (	scan_lines_per_image, 
							max_height , 
							height[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							intensity[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							scatter[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							reflectance[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							color[3*i*scan_lines_per_image:(3*i+3)*scan_lines_per_image], 
							kernel_exec_time,
							total_exec_time)
				else:
					yield (scan_lines_per_image, 
							max_height , 
							height[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							intensity[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							scatter[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							reflectance[i*scan_lines_per_image:(i+1)*scan_lines_per_image], 
							color[3*i*scan_lines_per_image:(3*i+3)*scan_lines_per_image],  
							None,
							None) # Do not report the same process time multiple times
		else:
			raise ValueError("Unknown scan data type", recv_type)
					
	
	def issueRequest(self, requestType, data):
		# send request
		self.sendMessage(requestType, data)
		
		# flush incoming buffer until response message is received
		recv_type = None
		while (recv_type != self.MessageType.RESPONSE):
			(recv_type, data) = self.getMessage()

		decoded_data = struct.unpack('BB', data[0:2])
		recv_type = int(decoded_data[0])
		ret_code  = int(decoded_data[1])

		data = data[2:]
		
		assert(recv_type == requestType)

		return (ret_code, data)
