#ifndef CONFIG_EVT_BROKER_H__
#define CONFIG_EVT_BROKER_H__

#ifdef __cplusplus
extern "C"
{
#endif


#include "ret_code.h"
#include "tcp_server.h"

/** @brief Validates and redirects configuration events to the correct module.
 *
 *  @details The validation consists of converting the TCP message payload into the correct structs and function calls.
 *           The module returns a message indicating the result of the configuration.
 *
 *  @param[in]  p_evt                   The TCP event to be decoded and forwarded.
 *  @param[out] p_response              The response message with its payload.
 *
 *  @return RET_SUCCESS                 If the configuration was decoded and applied successfully.
 *  @return RET_ERROR_INVALID_LENGTH    If the configuration message has an invalid length.
 *  @return RET_ERROR_NULL              If a NULL parameter was provided.
 *  @return                             Some other error if configuration failed.
 */
ret_code_t config_evt_broker_on_tcp_evt(tcp_evt_t const * p_evt,
                                        message_t * p_response);

#ifdef __cplusplus
}
#endif

#endif // CONFIG_EVT_BROKER_H__
