extern "C" {
#include "cuda_device_info.h"
}

#include "log.h"

void cuda_device_info_print(void)
{
    cudaError_t cuda_error;
    struct cudaDeviceProp props;
    int device;

    cuda_error = cudaGetDevice(&device);
    if (cuda_error != cudaSuccess)
    {
        LOG("Could not obtain cuda device\n");
        return;
    }

    cuda_error = cudaGetDeviceProperties(&props,device);
    if (cuda_error != cudaSuccess)
    {
        LOG("Could not obtain cuda device properties\n");
        return;
    }

    LOG("name[256];                  is %s\n",props.name                       );
    LOG("totalGlobalMem;             is %zu\n",props.totalGlobalMem             );
    LOG("sharedMemPerBlock;          is %zu\n",props.sharedMemPerBlock          );
    LOG("regsPerBlock;               is %d\n",props.regsPerBlock               );
    LOG("warpSize;                   is %d\n",props.warpSize                   );
    LOG("memPitch;                   is %zu\n",props.memPitch                   );
    LOG("maxThreadsPerBlock;         is %d\n",props.maxThreadsPerBlock         );
    LOG("maxThreadsDim[3];           is %d, %d, %d\n",props.maxThreadsDim[0],props.maxThreadsDim[1],props.maxThreadsDim[2]);
    LOG("maxGridSize[3];             is %d, %d, %d\n",props.maxGridSize[0],props.maxGridSize[1],props.maxGridSize[2]);
    LOG("clockRate;                  is %d\n",props.clockRate                  );
    LOG("totalConstMem;              is %zu\n",props.totalConstMem              );
    LOG("major;                      is %d\n",props.major                      );
    LOG("minor;                      is %d\n",props.minor                      );
    LOG("textureAlignment;           is %zu\n",props.textureAlignment           );
    LOG("texturePitchAlignment;      is %zu\n",props.texturePitchAlignment      );
    LOG("deviceOverlap;              is %d\n",props.deviceOverlap              );
    LOG("multiProcessorCount;        is %d\n",props.multiProcessorCount        );
    LOG("kernelExecTimeoutEnabled;   is %d\n",props.kernelExecTimeoutEnabled   );
    LOG("integrated;                 is %d\n",props.integrated                 );
    LOG("canMapHostMemory;           is %d\n",props.canMapHostMemory           );
    LOG("computeMode;                is %d\n",props.computeMode                );
    LOG("maxTexture1D;               is %d\n",props.maxTexture1D               );
    LOG("maxTexture1DMipmap;         is %d\n",props.maxTexture1DMipmap         );
    LOG("maxTexture1DLinear;         is %d\n",props.maxTexture1DLinear         );
    LOG("maxTexture2D[2];            is %d, %d\n",props.maxTexture2D[0],props.maxTexture2D[1]);
    LOG("maxTexture2DMipmap[2];      is %d, %d\n",props.maxTexture2DMipmap[0],props.maxTexture2DMipmap[1]);
    LOG("maxTexture2DLinear[3];      is %d, %d, %d\n",props.maxTexture2DLinear[0],props.maxTexture2DLinear[1],props.maxTexture2DLinear[2]);
    LOG("maxTexture2DGather[2];      is %d, %d\n",props.maxTexture2DGather[0],props.maxTexture2DGather[1]);
    LOG("maxTexture3D[3];            is %d, %d, %d\n",props.maxTexture3D[0],props.maxTexture3D[1],props.maxTexture3D[2]);
    LOG("maxTexture3DAlt[3];         is %d, %d, %d\n",props.maxTexture3DAlt[0],props.maxTexture3DAlt[1],props.maxTexture3DAlt[2]);
    LOG("maxTextureCubemap;          is %d\n",props.maxTextureCubemap          );
    LOG("maxTexture1DLayered[2];     is %d, %d\n",props.maxTexture1DLayered[0],props.maxTexture1DLayered[1]);
    LOG("maxTexture2DLayered[3];     is %d, %d, %d\n",props.maxTexture2DLayered[0],props.maxTexture2DLayered[1],props.maxTexture2DLayered[2]);
    LOG("maxTextureCubemapLayered[2];is %d, %d\n",props.maxTextureCubemapLayered[0],props.maxTextureCubemapLayered[1]);
    LOG("maxSurface1D;               is %d\n",props.maxSurface1D               );
    LOG("maxSurface2D[2];            is %d, %d\n",props.maxSurface2D[0],props.maxSurface2D[1]);
    LOG("maxSurface3D[3];            is %d, %d, %d\n",props.maxSurface3D[0],props.maxSurface3D[1],props.maxSurface3D[2]);
    LOG("maxSurface1DLayered[2];     is %d, %d\n",props.maxSurface1DLayered[0],props.maxSurface1DLayered[1]);
    LOG("maxSurface2DLayered[3];     is %d, %d, %d\n",props.maxSurface2DLayered[0],props.maxSurface2DLayered[1],props.maxSurface2DLayered[2]);
    LOG("maxSurfaceCubemap;          is %d\n",props.maxSurfaceCubemap          );
    LOG("maxSurfaceCubemapLayered[2];is %d, %d\n",props.maxSurfaceCubemapLayered[0], props.maxSurfaceCubemapLayered[1]);
    LOG("surfaceAlignment;           is %zu\n",props.surfaceAlignment           );
    LOG("concurrentKernels;          is %d\n",props.concurrentKernels          );
    LOG("ECCEnabled;                 is %d\n",props.ECCEnabled                 );
    LOG("pciBusID;                   is %d\n",props.pciBusID                   );
    LOG("pciDeviceID;                is %d\n",props.pciDeviceID                );
    LOG("pciDomainID;                is %d\n",props.pciDomainID                );
    LOG("tccDriver;                  is %d\n",props.tccDriver                  );
    LOG("asyncEngineCount;           is %d\n",props.asyncEngineCount           );
    LOG("unifiedAddressing;          is %d\n",props.unifiedAddressing          );
    LOG("memoryClockRate;            is %d\n",props.memoryClockRate            );
    LOG("memoryBusWidth;             is %d\n",props.memoryBusWidth             );
    LOG("l2CacheSize;                is %d\n",props.l2CacheSize                );
    LOG("maxThreadsPerMultiProcessor;is %d\n",props.maxThreadsPerMultiProcessor);
    LOG("streamPrioritiesSupported;  is %d\n",props.streamPrioritiesSupported  );
    LOG("globalL1CacheSupported;     is %d\n",props.globalL1CacheSupported     );
    LOG("localL1CacheSupported;      is %d\n",props.localL1CacheSupported      );
    LOG("sharedMemPerMultiprocessor; is %zu\n",props.sharedMemPerMultiprocessor );
    LOG("regsPerMultiprocessor;      is %d\n",props.regsPerMultiprocessor      );
    LOG("managedMemory;              is %d\n",props.managedMemory              );
    LOG("isMultiGpuBoard;            is %d\n",props.isMultiGpuBoard            );
    LOG("multiGpuBoardGroupID;       is %d\n",props.multiGpuBoardGroupID       );
}
