#include "gpio.h"
#include "log.h"

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>


#define SYSFS_GPIO_DIR "/sys/class/gpio"
#define MAX_BUF 64


static int        m_pin_fd = -1;
static gpio_pin_t m_pin;

ret_code_t gpio_init_output(gpio_pin_t pin)
{
    if (m_pin_fd != -1)
    {
        gpio_deinit_output(pin);
    }


    // Export the GPIO pin
    int fileDescriptor, length;
    char commandBuffer[MAX_BUF];

    fileDescriptor = open(SYSFS_GPIO_DIR "/export", O_WRONLY);
    if (fileDescriptor < 0)
    {
        char errorBuffer[128] ;
        snprintf(errorBuffer,sizeof(errorBuffer), "gpioExport unable to open gpio%d", pin) ;
        perror(errorBuffer);
        return RET_ERROR_OTHER;
    }

    length = snprintf(commandBuffer, sizeof(commandBuffer), "%d", pin);
    if (write(fileDescriptor, commandBuffer, length) != length)
    {
        perror("gpioExport");
        return RET_ERROR_OTHER ;
    }

    close(fileDescriptor);

    // Set direction

    snprintf(commandBuffer, sizeof(commandBuffer), SYSFS_GPIO_DIR  "/gpio%d/direction", pin);

    fileDescriptor = open(commandBuffer, O_WRONLY);
    if (fileDescriptor < 0)
    {
        char errorBuffer[128] ;
        snprintf(errorBuffer,sizeof(errorBuffer), "gpioSetDirection unable to open gpio%d",pin) ;
        perror(errorBuffer);
        return RET_ERROR_OTHER;
    }

    if (write(fileDescriptor, "out", 4) != 4) {
        perror("gpioSetDirection") ;
        return RET_ERROR_OTHER ;
    }

    close(fileDescriptor);

    // open the gpio file

    snprintf(commandBuffer, sizeof(commandBuffer), SYSFS_GPIO_DIR "/gpio%d/value", pin);

    m_pin_fd = open(commandBuffer, O_WRONLY);
    if (m_pin_fd < 0)
    {
        char errorBuffer[128] ;
        snprintf(errorBuffer,sizeof(errorBuffer), "gpioOpen unable to open gpio%d", pin) ;
        perror(errorBuffer);

        return RET_ERROR_OTHER;
    }

    m_pin = pin;

    return RET_SUCCESS;
}


ret_code_t gpio_deinit_output(gpio_pin_t pin)
{
    if (m_pin_fd != -1)
    {
        close(m_pin_fd);
    }

    // Unexport the GPIO pin
    int fileDescriptor, length;
    char commandBuffer[MAX_BUF];

    fileDescriptor = open(SYSFS_GPIO_DIR "/unexport", O_WRONLY);
    if (fileDescriptor < 0)
    {
        char errorBuffer[128] ;
        snprintf(errorBuffer,sizeof(errorBuffer), "gpioUnExport unable to open gpio%d", pin) ;
        perror(errorBuffer);
        return RET_ERROR_OTHER;
    }

    length = snprintf(commandBuffer, sizeof(commandBuffer), "%d", pin);
    if (write(fileDescriptor, commandBuffer, length) != length)
    {
        perror("gpioUnExport");
        return RET_ERROR_OTHER ;
    }

    // set the gpio pin to an invalid state
    m_pin_fd = -1;

    return RET_SUCCESS;
}

ret_code_t gpio_pin_set(gpio_pin_t pin, bool high)
{
    if (pin != m_pin)
    {
        LOG_DEBUG("Pin is not initialized\n");
        return RET_ERROR_INVALID_STATE;
    }

    if (m_pin_fd == -1)
    {
        LOG("Pin is not initialized");
        return RET_ERROR_INVALID_STATE;
    }

    char commandBuffer[MAX_BUF];
    snprintf(commandBuffer, sizeof(commandBuffer), SYSFS_GPIO_DIR "/gpio%d/value", pin);

    if (high) {
        if (write(m_pin_fd, "1", 2) != 2) {
            perror("gpioSetValue") ;
            return RET_ERROR_OTHER ;
        }
    }
    else {
        if (write(m_pin_fd, "0", 2) != 2) {
            perror("gpioSetValue") ;
            return RET_ERROR_OTHER ;
        }
    }

    return RET_SUCCESS;
}
