#ifndef STATE_MACHINE_H__
#define STATE_MACHINE_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "ret_code.h"
#include "tcp_server.h"


/** @brief Definitions of the system states. */
typedef enum
{
    STATE_IDLE     = 0x01,  /**< The state indicating that the system is IDLE, that is the camera is deactivated and is not waiting for configuration data. */
    STATE_SCANNING = 0x02,  /**< The system is scanning frames from the camera and transmitting data to the host. */
    STATE_CONFIG   = 0x03   /**< The system is in configuration mode. In this mode the options used in the scanning state can be modified. */
} state_t;


/** @brief  Initializes the state machine
 *
 *  @details If the state machine is successfully initialized, the state is initialized to @ref STATE_IDLE.
 *
 *  @return RET_SUCCESS     If the state machine was successfully initialized.
 */
ret_code_t state_machine_init(void);


/** @brief Handler function for TCP events.
 *
 *  @details    When the connection state changes, the system returns to the idle state.
 *              Otherwise the messages are forwarded to the corresponding modules.
 *              In case that the TCP event was a request, a response is sent.
 *
 *  @param[in]  p_evt   Pointer to the TCP event to be handled
 */
void state_machine_on_tcp_evt(tcp_evt_t const * p_evt);


#ifdef __cplusplus
}
#endif

#endif // STATE_MACHINE_H__
