#ifndef APP_ERROR_H__
#define APP_ERROR_H__

#include <stdlib.h>
#include "log.h"
#include "ret_code.h"

#define APP_ERROR_CHECK(ret_code)                               \
    do {                                                        \
        if (ret_code != RET_SUCCESS)                       \
        {                                                       \
            error_handler(ret_code, __FILE__, __LINE__);        \
        }                                                       \
    } while (0)

#define APP_ERROR_HANDLER(ret_code) APP_ERROR_CHECK(ret_code)

#define RETURN_ON_ERROR(ret_code)           \
    do {                                    \
        if (ret_code != RET_SUCCESS)   \
        {                                   \
            return ret_code;                \
        }                                   \
    } while (0)


static inline void error_handler(ret_code_t ret_code,
                                 char * filename,
                                 int line_num)
{
    LOG("Error handler:\nGot error: %d\nIn file %s\nLine %d\n", ret_code, filename, line_num);
    exit(ret_code);
}

#endif // APP_ERROR_H__
